#include "sceneLoader.h"
#include "renderingData.h"
#include "renderingMethods.h"

extern Scene *scene;
extern RenderingData *data;

// These functions are in volumetricShadows.cpp
void DrawFogFrame( FrameBuffer *frame, GLSLProgram *shader );
void StoreDistToFrontAndBackFogVolumes( FrameBuffer *frame );


void Display_WithBasicVolumetricShadows( void )
{
	// The colored and white lights need different intensities and scattering coefficients
	//    to get a similar effect.  This approximates this so that the same value specified
	//    in the file gives about the same contrast between illuminated & shadowed regions.
	float oldLightIntensity = scene->GetLightIntensityModifier();
	scene->SetLightIntensityModifier( 3*oldLightIntensity );
	float oldExtinctionCoef = scene->GetFogExtinctionCoef();
	scene->SetFogExtinctionCoef( 2*oldExtinctionCoef );

	// Compute a shadow map
	scene->CreateShadowMap( data->fbo->shadowMap, 
		                    data->param->shadowMatrixTrans, 
							0, // Light Number
							*(data->param->shadowMapBias) );

	// Draw the scene without shadows
	data->fbo->volShad_firstPass->BindBuffer();
	data->fbo->volShad_firstPass->ClearBuffers();
		glLoadIdentity();
		scene->LookAtMatrix();
		scene->SetupEnabledLightsWithCurrentModelview();
		scene->Draw( MATL_FLAGS_OMNIDIRECTIONSHADOWS );
	data->fbo->volShad_firstPass->UnbindBuffer();

	// Store positions of the front and back of the shadow volumes, so we can march 
	//    through the right regions of space.
	StoreDistToFrontAndBackFogVolumes( data->fbo->volShad_shadowVolFrontBack );

	// Compute the current frame's view-dependant airlight texture
	//    Note: Currently, to make the demo simpler this is not being used.  We picked
	//          the Hoffman and Preetham model for scattering here, as it is more
	//          straightforward to use.  This can be reenabled by uncommenting these
	//          lines and making the changes discussed in the shader
	//                  computeLowResScattering_PointLightFrontToBackVol.frag.glsl
	//float lightPos[4];
	//glGetLightfv( GL_LIGHT0, GL_POSITION, lightPos );
	//CreatePerFrameAirlightTexture( data->fbo->fog_perFrameFTex, lightPos, scene->GetCamera()->GetFar() );

	// Render a low resolution version of the fog inscattering (the key to this algorithm)
	DrawFogFrame( data->fbo->volShad_lowResFog, data->shader->volShad_computeBasicLowResScattering );

	// Combine the low resolution scattering image, and the first pass (image of scene w/o scattering)
	data->fbo->mainWin->BindBuffer();
	data->fbo->mainWin->ClearBuffers();
		data->shader->volShad_lowResPlusGeometry->EnableShader();
		data->shader->volShad_lowResPlusGeometry->SetTextureBinding( "lightIntegral", GL_TEXTURE0 );
		data->shader->volShad_lowResPlusGeometry->SetParameter( "extinctionCoef", scene->GetFogExtinctionCoef() );
		data->fbo->volShad_lowResFog->DisplayAsFullScreenTexture( FBO_COLOR0, GL_NEAREST );
		data->shader->volShad_lowResPlusGeometry->DisableShader();
	data->fbo->mainWin->UnbindBuffer();

	// Reset the light intensity & extiction coef, since we adjusted them at the start of this routine
	scene->SetLightIntensityModifier( oldLightIntensity );
	scene->SetFogExtinctionCoef( oldExtinctionCoef );
}