/***************************************************************************/
/* renderingData.h                                                         */
/* -----------------------                                                 */
/*                                                                         */
/* Stores a "class" that encapsulates global data used by the various      */
/*     rendering techniques described in this directory.  The internals of */
/*     this class are initialized by calling InitializeRenderingData()     */
/*     defined in initializeRenderingData.cpp.                             */
/*                                                                         */
/* Chris Wyman (02/23/2008)                                                */
/***************************************************************************/

#ifndef __RENDERING_DATA_H__
#define __REDNERING_DATA_H__

class FrameBuffer;
class GLSLProgram;
class GLTexture;
class FrameBufferData;
class VertexBufferData;
class TextureData;
class ShaderData;
class ParameterData;
class OtherGLIDData;
class UIData;
class FrameRate;
class FrameGrab;

// Class encapsulating global rendering data
class RenderingData
{
public:
	FrameBufferData  *fbo;
	ShaderData       *shader;
	ParameterData    *param;
	TextureData      *tex;
	VertexBufferData *vbo;
	OtherGLIDData    *glID;
	UIData           *ui;
};



class FrameBufferData
{
public:
	// Framebuffers for simple rendering with shadow maps
	FrameBuffer *shadowMap, *mainWin;
	FrameBuffer *helpScreen;

	// Framebuffers for volumetric fog
	FrameBuffer *volShad_firstPass;     // Stores the first pass (scene w/o particpating media)
	FrameBuffer *volShad_lightVol;      // Stores the light view frustum
	FrameBuffer *volShad_lowResFog;     // Stores the computed fog (at low-ish resolution)
	FrameBuffer *fog_perFrameFTex;      // Our acceleration to the Sun et al. model
	FrameBuffer *volShad_shadowVolFrontBack;  // Stores front & back sides of the shadow volume
};


class ShaderData
{
public:

	// Shaders for volumetric shadows
	GLSLProgram *fog_CreatePerFrameAirlightTex, *volShad_storeLightVol; /* *volShad_finalRender; */
	GLSLProgram /* *volShad_computeLowResScattering, */ *volShad_lowResPlusGeometry;

	GLSLProgram *volShad_computeShadowVolFrontAndBack, *volShad_computeLowResScatteringWithShadVol;
	GLSLProgram *volShad_capShadowVol;

	GLSLProgram *volShad_test;

	GLSLProgram *volShad_computeBasicLowResScattering;
};


class ParameterData
{
public:
	// Parameters for simple rendering with shadow maps
	float shadowMatrixTrans[16];
	bool *usingShadows;
	float *shadowMapBias;

	// Parameters for volumetric shadows
	float fog_FTexRange[4];
	float *lightFOV;
};

class TextureData
{
public:
	// Spotlight texture.  This is typically needed only in the Scene class,
	//    but sometimes (e.g., with caustics) it may be needed elsewhere.
	GLTexture *spotlight, *lowResSpot, *blurrySpot;

	GLTexture *iowaLogo, *washColLogo;
};

class VertexBufferData
{
public:
};

class OtherGLIDData
{
public:
	// Occlusion Query IDs
	GLuint primCountQuery;

	// A texture ID for the airlight scattering texture.  
	GLuint airlightTexture;
};

class UIData
{
public:
	bool captureScreen, displayHelp;
	bool *animation;
	int *displayID;
	FrameRate *frameSpeed;
	FrameGrab *frameGrab;
};



#endif

