/***************************************************************************/
/* renderingMethods.h                                                      */
/* -----------------------                                                 */
/*                                                                         */
/* Has prototypes and other data needed to use the various rendering       */
/*     techniques (i.e., DisplayCallback()'s) located in the .cpp files    */
/*     in this directory.                                                  */
/*                                                                         */
/* Chris Wyman (02/01/2008)                                                */
/***************************************************************************/


#ifndef __REDNERINGMETHODS_H__
#define __REDNERINGMETHODS_H__

class FrameBuffer;

// Display routines.  Currently, these are called via a case statement
//   inside DisplayCallback() inside sceneLoader.cpp
void Display_WithRefraction( void );				         // See: basicRefraction.cpp
void Display_WithSimpleRefractiveCaustics( void );	         // See: simpleCausticMaps.cpp
void Display_WithHierarchicalRefractiveCaustics( void );     // See: hierarchicalCausticMaps.cpp
void Display_WithVolumetricShadows( void );                  // See: volumetricShadows.cpp
void Display_WithBasicVolumetricShadows( void );

// The data structure in "renderingData.h" stores various data needed
//    by the rendering techniques in this directory.  This function 
//    initializes all this data.
void InitializeRenderingData( void );					     // See: initializeRenderingData.cpp

// Some functions to create and display a help screen
void SetupHelpScreen( FrameBuffer *helpFB );
void DisplayHelpScreen( FrameBuffer *helpFB );

// Utility routines used by multiple rendering techniques
void SetupAirlightModel_FTexture( GLuint airlightID );            // See: utilRoutines.cpp
void CreatePerFrameAirlightTexture( FrameBuffer *frame,           // See: utilRoutines.cpp
								    float *eyeSpaceLightPos, 
									float maxDist = 50.0 );
void DrawLightVolume( FrameBuffer *frame,                         // See: utilRoutines.cpp
					  unsigned int lightNum = 0 );


#endif

