/***************************************************************************/
/* volumetricShadows2.cpp                                                  */
/* -----------------------                                                 */
/*                                                                         */
/* experimental stuff...                                                   */
/*                                                                         */
/* Chris Wyman (02/23/2008)                                                */
/***************************************************************************/


#include "sceneLoader.h"
#include "renderingData.h"
#include "renderingMethods.h"

extern Scene *scene;
extern RenderingData *data;

// When we compute scattering at every pixel, we don't need to render scene-
//   specific geometry (that's already been rendered in the first pass).
//   Instead, we'll just take in textures representing the first pass and
//   perhaps specifics related to the fog model we're using, and render
//   a full-screen quad (since there's fog at every pixel).
// This function is left over from our research implementation, where we
//   did this using a variety of shaders using different tests, but they
//   all had to setup the same variables, use the same textures, and render
//   the full-screen quad the same way.  It takes in a shader to use
//   and an appropriate output buffer and runs the fog calculations.
void DrawFogFrame( FrameBuffer *frame, GLSLProgram *shader )
{
	frame->BindBuffer();
	frame->ClearBuffers();
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0,1,0,1);
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	shader->EnableShader();
	shader->SetParameter( "lightIntensity", scene->GetLightIntensityModifier() );
	shader->SetParameter( "useAlternateFog", 1 );
	shader->SetParameter( "extinctionCoef", scene->GetFogExtinctionCoef() );
	shader->SetParameter( "farDist", scene->GetCamera()->GetFar() );
	shader->SetParameter( "tan_fovy_2", tan( scene->GetCamera()->GetFovy() * M_PI / 360.0f ) );
	shader->SetParameter( "sin_fovy_2", sin( scene->GetCamera()->GetFovy() * M_PI / 360.0f ) );
	shader->SetParameter( "imgSize", frame->GetWidth(), frame->GetHeight() );
	glActiveTexture( GL_TEXTURE3 );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE );
	glBegin( GL_QUADS );
		glTexCoord2f(0,0);	glVertex2f(0,0);
		glTexCoord2f(1,0);	glVertex2f(1,0);
		glTexCoord2f(1,1);	glVertex2f(1,1);
		glTexCoord2f(0,1);	glVertex2f(0,1);
	glEnd();
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE );
	glActiveTexture( GL_TEXTURE0 );
	shader->DisableShader();
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
	frame->UnbindBuffer();
}


// Stores the frontmost and backmost sides of the shadow volume
void StoreDistToFrontAndBackFogVolumes( FrameBuffer *frame )
{
	frame->BindBuffer();
	frame->ClearBuffers();
	glLoadIdentity();
	scene->LookAtMatrix();
	scene->SetupEnabledLightsWithCurrentModelview();

	// This pass goes over the dragon, finds silhouette edges (from light POV), 
	//    extrudes them into shadow volumes, and finds the closest and furthest 
	//    from the eye (using a geometry shader to output to two different layers
	//    for "closest" and "furthest")
	data->shader->volShad_computeShadowVolFrontAndBack->EnableShader();
	data->shader->volShad_computeShadowVolFrontAndBack->BindAndEnableTexture( "sceneColorAndDist", 
		 data->fbo->volShad_firstPass->GetColorTextureID(0), GL_TEXTURE0, GL_TEXTURE_2D );
	scene->DrawOnly( OBJECT_FLAGS_ISFOREGROUND | OBJECT_FLAGS_ALLOWDRAWEDGESONLY, MATL_FLAGS_NONE, OBJECT_OPTION_USE_LOWRES, true );
	data->shader->volShad_computeShadowVolFrontAndBack->DisableTexture();
	data->shader->volShad_computeShadowVolFrontAndBack->DisableShader();

	// The shadow volumes have holes on the end, so this caps the volume by drawing 
	//    a copy of the object at the two ends of the volume.  We found it is sufficient to
	//    use a low-resolution version of the object for this capping.
	data->shader->volShad_capShadowVol->EnableShader();
	data->shader->volShad_capShadowVol->BindAndEnableTexture( "sceneColorAndDist", 
		 data->fbo->volShad_firstPass->GetColorTextureID(0), GL_TEXTURE0, GL_TEXTURE_2D );
	scene->DrawOnly( OBJECT_FLAGS_ISFOREGROUND, MATL_FLAGS_NONE, OBJECT_OPTION_USE_LOWRES, true );
	data->shader->volShad_capShadowVol->DisableTexture();
	data->shader->volShad_capShadowVol->DisableShader();
	
	frame->UnbindBuffer();
}


void Display_WithVolumetricShadows( void )
{
	// Compute a shadow map
	scene->CreateShadowMap( data->fbo->shadowMap, 
		                    data->param->shadowMatrixTrans, 
							0, // Light Number
							*(data->param->shadowMapBias) );

	// Store the region of the scene inside the spotlight volume
	DrawLightVolume( data->fbo->volShad_lightVol, 0 );

	// Draw the scene without shadows
	data->fbo->volShad_firstPass->BindBuffer();
	data->fbo->volShad_firstPass->ClearBuffers();
		glLoadIdentity();
		scene->LookAtMatrix();
		scene->SetupEnabledLightsWithCurrentModelview();
		scene->Draw( MATL_FLAGS_USESHADOWMAP );
	data->fbo->volShad_firstPass->UnbindBuffer();

	// Store positions of the front of the fog volume
	StoreDistToFrontAndBackFogVolumes( data->fbo->volShad_shadowVolFrontBack );

	// Compute the current frame's view-dependant airlight texture
	//    Note: Currently, to make the demo simpler this is not being used.  We picked
	//          the Hoffman and Preetham model for scattering here, as it is more
	//          straightforward to use.  This can be reenabled by uncommenting these
	//          lines and making the changes discussed in the shader
	//                  createLowResScatteringTextureUsingShadowVolume.frag.glsl
	//float lightPos[4];
	//glGetLightfv( GL_LIGHT0, GL_POSITION, lightPos );
	//CreatePerFrameAirlightTexture( data->fbo->fog_perFrameFTex, lightPos, scene->GetCamera()->GetFar() );

	// Render a low resolution version of the fog inscattering
	DrawFogFrame( data->fbo->volShad_lowResFog, data->shader->volShad_computeLowResScatteringWithShadVol );

	// Combine the low resolution scattering image, and the first pass (image of scene w/o scattering)
	data->fbo->mainWin->BindBuffer();
	data->fbo->mainWin->ClearBuffers();
		data->shader->volShad_lowResPlusGeometry->EnableShader();
		data->shader->volShad_lowResPlusGeometry->SetTextureBinding( "lightIntegral", GL_TEXTURE0 );
		data->shader->volShad_lowResPlusGeometry->SetParameter( "extinctionCoef", scene->GetFogExtinctionCoef() );
		data->fbo->volShad_lowResFog->DisplayAsFullScreenTexture( FBO_COLOR0, GL_NEAREST );
		data->shader->volShad_lowResPlusGeometry->DisableShader();
	data->fbo->mainWin->UnbindBuffer();
}


